[data-inspector-highlight] {
  outline: 2px solid #7158f1 !important;
  outline-offset: -1px;
}

#inspector-focus {
  position: absolute;
  z-index: 100;
  display: block;
  box-shadow: 0 0 1px 1px #0001a;
  animation-name: inspector-focus-animation;
  animation-duration: .3s;
  animation-timing-function: linear;
  animation-delay: .1s;
  animation-iteration-count: 1;
  pointer-events: none;
}

@keyframes inspector-focus-animation {
  0% {
    box-shadow: 0 0 0 0 rgba(255, 255, 255, .8);
  }
  50% {
    box-shadow: 0 0 0 10px rgba(0, 0, 0, .4);
  }
  100% {
    box-shadow: 0 0 0 20px rgba(0, 0, 0, 0);
  }
}

#inspector-frame {
  position: fixed;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 100;
  box-shadow: inset 0 0 0 3px #7158f1;
  cursor: crosshair;
  pointer-events: none;
}

#inspector-frame::after {
  position: fixed;
  bottom: 0;
  left: 50%;
  margin-left: -150px;
  padding: 5px;
  width: 300px;
  background: #7158f1;
  color: white;
  /* content: 'My content should be specific to operation being run'; */
  text-align: center;
  font-size: 16px;
  opacity: 1.0;
}

#inspector-frame.picker::after {
  content: 'Click element to select it';
}

#inspector-frame.recorder::after {
  content: 'Click to record, ESC to stop recording';
}

