parcelRequire=function(e,r,t,n){var i,o="function"==typeof parcelRequire&&parcelRequire,u="function"==typeof require&&require;function f(t,n){if(!r[t]){if(!e[t]){var i="function"==typeof parcelRequire&&parcelRequire;if(!n&&i)return i(t,!0);if(o)return o(t,!0);if(u&&"string"==typeof t)return u(t);var c=new Error("Cannot find module '"+t+"'");throw c.code="MODULE_NOT_FOUND",c}p.resolve=function(r){return e[t][1][r]||r},p.cache={};var l=r[t]=new f.Module(t);e[t][0].call(l.exports,p,l,l.exports,this)}return r[t].exports;function p(e){return f(p.resolve(e))}}f.isParcelRequire=!0,f.Module=function(e){this.id=e,this.bundle=f,this.exports={}},f.modules=e,f.cache=r,f.parent=o,f.register=function(r,t){e[r]=[function(e,r){r.exports=t},{}]};for(var c=0;c<t.length;c++)try{f(t[c])}catch(e){i||(i=e)}if(t.length){var l=f(t[t.length-1]);"object"==typeof exports&&"undefined"!=typeof module?module.exports=l:"function"==typeof define&&define.amd?define(function(){return l}):n&&(this[n]=l)}if(parcelRequire=f,i)throw i;return f}({"W2+e":[function(require,module,exports) {
"use strict";var r=Object.getOwnPropertySymbols,t=Object.prototype.hasOwnProperty,e=Object.prototype.propertyIsEnumerable;function n(r){if(null==r)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(r)}function o(){try{if(!Object.assign)return!1;var r=new String("abc");if(r[5]="de","5"===Object.getOwnPropertyNames(r)[0])return!1;for(var t={},e=0;e<10;e++)t["_"+String.fromCharCode(e)]=e;if("0123456789"!==Object.getOwnPropertyNames(t).map(function(r){return t[r]}).join(""))return!1;var n={};return"abcdefghijklmnopqrst".split("").forEach(function(r){n[r]=r}),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},n)).join("")}catch(o){return!1}}module.exports=o()?Object.assign:function(o,c){for(var a,i,s=n(o),f=1;f<arguments.length;f++){for(var u in a=Object(arguments[f]))t.call(a,u)&&(s[u]=a[u]);if(r){i=r(a);for(var b=0;b<i.length;b++)e.call(a,i[b])&&(s[i[b]]=a[i[b]])}}return s};
},{}],"f/82":[function(require,module,exports) {
module.exports=function(o){return o&&"object"==typeof o&&"function"==typeof o.copy&&"function"==typeof o.fill&&"function"==typeof o.readUInt8};
},{}],"6rG3":[function(require,module,exports) {
"function"==typeof Object.create?module.exports=function(t,e){t.super_=e,t.prototype=Object.create(e.prototype,{constructor:{value:t,enumerable:!1,writable:!0,configurable:!0}})}:module.exports=function(t,e){t.super_=e;var o=function(){};o.prototype=e.prototype,t.prototype=new o,t.prototype.constructor=t};
},{}],"rH1J":[function(require,module,exports) {

var t,e,n=module.exports={};function r(){throw new Error("setTimeout has not been defined")}function o(){throw new Error("clearTimeout has not been defined")}function i(e){if(t===setTimeout)return setTimeout(e,0);if((t===r||!t)&&setTimeout)return t=setTimeout,setTimeout(e,0);try{return t(e,0)}catch(n){try{return t.call(null,e,0)}catch(n){return t.call(this,e,0)}}}function u(t){if(e===clearTimeout)return clearTimeout(t);if((e===o||!e)&&clearTimeout)return e=clearTimeout,clearTimeout(t);try{return e(t)}catch(n){try{return e.call(null,t)}catch(n){return e.call(this,t)}}}!function(){try{t="function"==typeof setTimeout?setTimeout:r}catch(n){t=r}try{e="function"==typeof clearTimeout?clearTimeout:o}catch(n){e=o}}();var c,s=[],l=!1,a=-1;function f(){l&&c&&(l=!1,c.length?s=c.concat(s):a=-1,s.length&&h())}function h(){if(!l){var t=i(f);l=!0;for(var e=s.length;e;){for(c=s,s=[];++a<e;)c&&c[a].run();a=-1,e=s.length}c=null,l=!1,u(t)}}function m(t,e){this.fun=t,this.array=e}function p(){}n.nextTick=function(t){var e=new Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)e[n-1]=arguments[n];s.push(new m(t,e)),1!==s.length||l||i(h)},m.prototype.run=function(){this.fun.apply(null,this.array)},n.title="browser",n.env={},n.argv=[],n.version="",n.versions={},n.on=p,n.addListener=p,n.once=p,n.off=p,n.removeListener=p,n.removeAllListeners=p,n.emit=p,n.prependListener=p,n.prependOnceListener=p,n.listeners=function(t){return[]},n.binding=function(t){throw new Error("process.binding is not supported")},n.cwd=function(){return"/"},n.chdir=function(t){throw new Error("process.chdir is not supported")},n.umask=function(){return 0};
},{}],"9JOR":[function(require,module,exports) {
var global = arguments[3];
var process = require("process");
var e=arguments[3],t=require("process"),r=/%[sdj%]/g;exports.format=function(e){if(!v(e)){for(var t=[],n=0;n<arguments.length;n++)t.push(i(arguments[n]));return t.join(" ")}n=1;for(var o=arguments,s=o.length,u=String(e).replace(r,function(e){if("%%"===e)return"%";if(n>=s)return e;switch(e){case"%s":return String(o[n++]);case"%d":return Number(o[n++]);case"%j":try{return JSON.stringify(o[n++])}catch(t){return"[Circular]"}default:return e}}),c=o[n];n<s;c=o[++n])h(c)||!z(c)?u+=" "+c:u+=" "+i(c);return u},exports.deprecate=function(r,n){if(j(e.process))return function(){return exports.deprecate(r,n).apply(this,arguments)};if(!0===t.noDeprecation)return r;var o=!1;return function(){if(!o){if(t.throwDeprecation)throw new Error(n);t.traceDeprecation?console.trace(n):console.error(n),o=!0}return r.apply(this,arguments)}};var n,o={};function i(e,t){var r={seen:[],stylize:u};return arguments.length>=3&&(r.depth=arguments[2]),arguments.length>=4&&(r.colors=arguments[3]),x(t)?r.showHidden=t:t&&exports._extend(r,t),j(r.showHidden)&&(r.showHidden=!1),j(r.depth)&&(r.depth=2),j(r.colors)&&(r.colors=!1),j(r.customInspect)&&(r.customInspect=!0),r.colors&&(r.stylize=s),p(r,e,r.depth)}function s(e,t){var r=i.styles[t];return r?"["+i.colors[r][0]+"m"+e+"["+i.colors[r][1]+"m":e}function u(e,t){return e}function c(e){var t={};return e.forEach(function(e,r){t[e]=!0}),t}function p(e,t,r){if(e.customInspect&&t&&D(t.inspect)&&t.inspect!==exports.inspect&&(!t.constructor||t.constructor.prototype!==t)){var n=t.inspect(r,e);return v(n)||(n=p(e,n,r)),n}var o=l(e,t);if(o)return o;var i=Object.keys(t),s=c(i);if(e.showHidden&&(i=Object.getOwnPropertyNames(t)),E(t)&&(i.indexOf("message")>=0||i.indexOf("description")>=0))return a(t);if(0===i.length){if(D(t)){var u=t.name?": "+t.name:"";return e.stylize("[Function"+u+"]","special")}if(O(t))return e.stylize(RegExp.prototype.toString.call(t),"regexp");if(w(t))return e.stylize(Date.prototype.toString.call(t),"date");if(E(t))return a(t)}var x,h="",b=!1,m=["{","}"];(d(t)&&(b=!0,m=["[","]"]),D(t))&&(h=" [Function"+(t.name?": "+t.name:"")+"]");return O(t)&&(h=" "+RegExp.prototype.toString.call(t)),w(t)&&(h=" "+Date.prototype.toUTCString.call(t)),E(t)&&(h=" "+a(t)),0!==i.length||b&&0!=t.length?r<0?O(t)?e.stylize(RegExp.prototype.toString.call(t),"regexp"):e.stylize("[Object]","special"):(e.seen.push(t),x=b?f(e,t,r,s,i):i.map(function(n){return g(e,t,r,s,n,b)}),e.seen.pop(),y(x,h,m)):m[0]+h+m[1]}function l(e,t){if(j(t))return e.stylize("undefined","undefined");if(v(t)){var r="'"+JSON.stringify(t).replace(/^"|"$/g,"").replace(/'/g,"\\'").replace(/\\"/g,'"')+"'";return e.stylize(r,"string")}return m(t)?e.stylize(""+t,"number"):x(t)?e.stylize(""+t,"boolean"):h(t)?e.stylize("null","null"):void 0}function a(e){return"["+Error.prototype.toString.call(e)+"]"}function f(e,t,r,n,o){for(var i=[],s=0,u=t.length;s<u;++s)$(t,String(s))?i.push(g(e,t,r,n,String(s),!0)):i.push("");return o.forEach(function(o){o.match(/^\d+$/)||i.push(g(e,t,r,n,o,!0))}),i}function g(e,t,r,n,o,i){var s,u,c;if((c=Object.getOwnPropertyDescriptor(t,o)||{value:t[o]}).get?u=c.set?e.stylize("[Getter/Setter]","special"):e.stylize("[Getter]","special"):c.set&&(u=e.stylize("[Setter]","special")),$(n,o)||(s="["+o+"]"),u||(e.seen.indexOf(c.value)<0?(u=h(r)?p(e,c.value,null):p(e,c.value,r-1)).indexOf("\n")>-1&&(u=i?u.split("\n").map(function(e){return"  "+e}).join("\n").substr(2):"\n"+u.split("\n").map(function(e){return"   "+e}).join("\n")):u=e.stylize("[Circular]","special")),j(s)){if(i&&o.match(/^\d+$/))return u;(s=JSON.stringify(""+o)).match(/^"([a-zA-Z_][a-zA-Z_0-9]*)"$/)?(s=s.substr(1,s.length-2),s=e.stylize(s,"name")):(s=s.replace(/'/g,"\\'").replace(/\\"/g,'"').replace(/(^"|"$)/g,"'"),s=e.stylize(s,"string"))}return s+": "+u}function y(e,t,r){return e.reduce(function(e,t){return 0,t.indexOf("\n")>=0&&0,e+t.replace(/\u001b\[\d\d?m/g,"").length+1},0)>60?r[0]+(""===t?"":t+"\n ")+" "+e.join(",\n  ")+" "+r[1]:r[0]+t+" "+e.join(", ")+" "+r[1]}function d(e){return Array.isArray(e)}function x(e){return"boolean"==typeof e}function h(e){return null===e}function b(e){return null==e}function m(e){return"number"==typeof e}function v(e){return"string"==typeof e}function S(e){return"symbol"==typeof e}function j(e){return void 0===e}function O(e){return z(e)&&"[object RegExp]"===A(e)}function z(e){return"object"==typeof e&&null!==e}function w(e){return z(e)&&"[object Date]"===A(e)}function E(e){return z(e)&&("[object Error]"===A(e)||e instanceof Error)}function D(e){return"function"==typeof e}function N(e){return null===e||"boolean"==typeof e||"number"==typeof e||"string"==typeof e||"symbol"==typeof e||void 0===e}function A(e){return Object.prototype.toString.call(e)}function J(e){return e<10?"0"+e.toString(10):e.toString(10)}exports.debuglog=function(e){if(j(n)&&(n=""),e=e.toUpperCase(),!o[e])if(new RegExp("\\b"+e+"\\b","i").test(n)){var r=t.pid;o[e]=function(){var t=exports.format.apply(exports,arguments);console.error("%s %d: %s",e,r,t)}}else o[e]=function(){};return o[e]},exports.inspect=i,i.colors={bold:[1,22],italic:[3,23],underline:[4,24],inverse:[7,27],white:[37,39],grey:[90,39],black:[30,39],blue:[34,39],cyan:[36,39],green:[32,39],magenta:[35,39],red:[31,39],yellow:[33,39]},i.styles={special:"cyan",number:"yellow",boolean:"yellow",undefined:"grey",null:"bold",string:"green",date:"magenta",regexp:"red"},exports.isArray=d,exports.isBoolean=x,exports.isNull=h,exports.isNullOrUndefined=b,exports.isNumber=m,exports.isString=v,exports.isSymbol=S,exports.isUndefined=j,exports.isRegExp=O,exports.isObject=z,exports.isDate=w,exports.isError=E,exports.isFunction=D,exports.isPrimitive=N,exports.isBuffer=require("./support/isBuffer");var R=["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"];function H(){var e=new Date,t=[J(e.getHours()),J(e.getMinutes()),J(e.getSeconds())].join(":");return[e.getDate(),R[e.getMonth()],t].join(" ")}function $(e,t){return Object.prototype.hasOwnProperty.call(e,t)}exports.log=function(){console.log("%s - %s",H(),exports.format.apply(exports,arguments))},exports.inherits=require("inherits"),exports._extend=function(e,t){if(!t||!z(t))return e;for(var r=Object.keys(t),n=r.length;n--;)e[r[n]]=t[r[n]];return e};
},{"./support/isBuffer":"f/82","inherits":"6rG3","process":"rH1J"}],"p+dr":[function(require,module,exports) {
var global = arguments[3];
var t=arguments[3],e=require("object-assign");function r(t,e){if(t===e)return 0;for(var r=t.length,n=e.length,i=0,o=Math.min(r,n);i<o;++i)if(t[i]!==e[i]){r=t[i],n=e[i];break}return r<n?-1:n<r?1:0}function n(e){return t.Buffer&&"function"==typeof t.Buffer.isBuffer?t.Buffer.isBuffer(e):!(null==e||!e._isBuffer)}var i=require("util/"),o=Object.prototype.hasOwnProperty,u=Array.prototype.slice,a="foo"===function(){}.name;function c(t){return Object.prototype.toString.call(t)}function f(e){return!n(e)&&("function"==typeof t.ArrayBuffer&&("function"==typeof ArrayBuffer.isView?ArrayBuffer.isView(e):!!e&&(e instanceof DataView||!!(e.buffer&&e.buffer instanceof ArrayBuffer))))}var s=module.exports=q,l=/\s*function\s+([^\(\s]*)\s*/;function p(t){if(i.isFunction(t)){if(a)return t.name;var e=t.toString().match(l);return e&&e[1]}}function g(t,e){return"string"==typeof t?t.length<e?t:t.slice(0,e):t}function E(t){if(a||!i.isFunction(t))return i.inspect(t);var e=p(t);return"[Function"+(e?": "+e:"")+"]"}function h(t){return g(E(t.actual),128)+" "+t.operator+" "+g(E(t.expected),128)}function y(t,e,r,n,i){throw new s.AssertionError({message:r,actual:t,expected:e,operator:n,stackStartFunction:i})}function q(t,e){t||y(t,!0,e,"==",s.ok)}function d(t,e,o,u){if(t===e)return!0;if(n(t)&&n(e))return 0===r(t,e);if(i.isDate(t)&&i.isDate(e))return t.getTime()===e.getTime();if(i.isRegExp(t)&&i.isRegExp(e))return t.source===e.source&&t.global===e.global&&t.multiline===e.multiline&&t.lastIndex===e.lastIndex&&t.ignoreCase===e.ignoreCase;if(null!==t&&"object"==typeof t||null!==e&&"object"==typeof e){if(f(t)&&f(e)&&c(t)===c(e)&&!(t instanceof Float32Array||t instanceof Float64Array))return 0===r(new Uint8Array(t.buffer),new Uint8Array(e.buffer));if(n(t)!==n(e))return!1;var a=(u=u||{actual:[],expected:[]}).actual.indexOf(t);return-1!==a&&a===u.expected.indexOf(e)||(u.actual.push(t),u.expected.push(e),b(t,e,o,u))}return o?t===e:t==e}function m(t){return"[object Arguments]"==Object.prototype.toString.call(t)}function b(t,e,r,n){if(null==t||null==e)return!1;if(i.isPrimitive(t)||i.isPrimitive(e))return t===e;if(r&&Object.getPrototypeOf(t)!==Object.getPrototypeOf(e))return!1;var o=m(t),a=m(e);if(o&&!a||!o&&a)return!1;if(o)return d(t=u.call(t),e=u.call(e),r);var c,f,s=A(t),l=A(e);if(s.length!==l.length)return!1;for(s.sort(),l.sort(),f=s.length-1;f>=0;f--)if(s[f]!==l[f])return!1;for(f=s.length-1;f>=0;f--)if(!d(t[c=s[f]],e[c],r,n))return!1;return!0}function v(t,e,r){d(t,e,!0)&&y(t,e,r,"notDeepStrictEqual",v)}function x(t,e){if(!t||!e)return!1;if("[object RegExp]"==Object.prototype.toString.call(e))return e.test(t);try{if(t instanceof e)return!0}catch(r){}return!Error.isPrototypeOf(e)&&!0===e.call({},t)}function S(t){var e;try{t()}catch(r){e=r}return e}function w(t,e,r,n){var o;if("function"!=typeof e)throw new TypeError('"block" argument must be a function');"string"==typeof r&&(n=r,r=null),o=S(e),n=(r&&r.name?" ("+r.name+").":".")+(n?" "+n:"."),t&&!o&&y(o,r,"Missing expected exception"+n);var u="string"==typeof n,a=!t&&o&&!r;if((!t&&i.isError(o)&&u&&x(o,r)||a)&&y(o,r,"Got unwanted exception"+n),t&&o&&r&&!x(o,r)||!t&&o)throw o}function O(t,e){t||y(t,!0,e,"==",O)}s.AssertionError=function(t){this.name="AssertionError",this.actual=t.actual,this.expected=t.expected,this.operator=t.operator,t.message?(this.message=t.message,this.generatedMessage=!1):(this.message=h(this),this.generatedMessage=!0);var e=t.stackStartFunction||y;if(Error.captureStackTrace)Error.captureStackTrace(this,e);else{var r=new Error;if(r.stack){var n=r.stack,i=p(e),o=n.indexOf("\n"+i);if(o>=0){var u=n.indexOf("\n",o+1);n=n.substring(u+1)}this.stack=n}}},i.inherits(s.AssertionError,Error),s.fail=y,s.ok=q,s.equal=function(t,e,r){t!=e&&y(t,e,r,"==",s.equal)},s.notEqual=function(t,e,r){t==e&&y(t,e,r,"!=",s.notEqual)},s.deepEqual=function(t,e,r){d(t,e,!1)||y(t,e,r,"deepEqual",s.deepEqual)},s.deepStrictEqual=function(t,e,r){d(t,e,!0)||y(t,e,r,"deepStrictEqual",s.deepStrictEqual)},s.notDeepEqual=function(t,e,r){d(t,e,!1)&&y(t,e,r,"notDeepEqual",s.notDeepEqual)},s.notDeepStrictEqual=v,s.strictEqual=function(t,e,r){t!==e&&y(t,e,r,"===",s.strictEqual)},s.notStrictEqual=function(t,e,r){t===e&&y(t,e,r,"!==",s.notStrictEqual)},s.throws=function(t,e,r){w(!0,t,e,r)},s.doesNotThrow=function(t,e,r){w(!1,t,e,r)},s.ifError=function(t){if(t)throw t},s.strict=e(O,s,{equal:s.strictEqual,deepEqual:s.deepStrictEqual,notEqual:s.notStrictEqual,notDeepEqual:s.notDeepStrictEqual}),s.strict.strict=s.strict;var A=Object.keys||function(t){var e=[];for(var r in t)o.call(t,r)&&e.push(r);return e};
},{"object-assign":"W2+e","util/":"9JOR"}],"WRAk":[function(require,module,exports) {
"use strict";var e=t(require("assert"));function t(e){return e&&e.__esModule?e:{default:e}}var n=function(){return(n=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n])Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e}).apply(this,arguments)},r=["name","title","id","for","href","class"],i=!1;!function(){function t(e){return e.nodeType===Node.ELEMENT_NODE}var a={_getIndex:function(e){var n=!1,r=0,i=0;if(!e.parentNode)return 0;for(var a=e.parentNode.childNodes,c=0;c<a.length;c++){a[c]===e&&(n=!0);var u=a[c];t(u)&&u.tagName===e.tagName&&(r+=1,i=n?i:i+1)}return r>1?i+1:0},_buildAttributes:function(e,t){return t.map(function(t){var n,r;return(r="className"===t?e.className.length>0?e.className.split(" "):null:"index"===t?1:e.getAttribute(t))?((n={})[""+t]=r,n):null}).filter(function(e){return e})},build:function(e,t,n){var r;if(!e)return n;if(!e.parentNode)return n;if(e.nodeType===Node.DOCUMENT_NODE)return n;var i=this._buildAttributes(e,t);return n.push(((r={})[""+e.tagName.toLowerCase()]=i,r)),this.build(e.parentNode,t,n)}},c={build:function(e,t,n){var r=this,i=e[0],a=Object.keys(i)[0],c="/"+i[a].reduce(function(e,t){return""===e?r._getSubpath(e,t,a):e},"");if(!t)return c;if(this._found(["@id","@for"],c))return c;if(this._found(["@name"],c)&&this._found(["select"],n))return c;var u=this._getIndex(c,t),l=u.count,o=u.index;return l>1&&o>1?"xpath=("+c+")["+o+"]":c},_found:function(e,t){return e.some(function(e){return t.includes(e)})},_getIndex:function(e,t){for(var n,r=1,i=1,a=document.evaluate("."+e,document.body,null,XPathResult.ORDERED_NODE_ITERATOR_TYPE,null);n=a.iterateNext();)n===t&&(r=i),i+=1;return{count:i,index:r}},_getSubpath:function(e,t,n){return null!=t.for?"/"+n+'[@for="'+t.for+'"]':null!=t.class&&"number"!=typeof t.class&&t.class.length>0?"/"+n+'[@class="'+t.class+'"]':null!=t.title?"/"+n+'[@title="'+t.title+'"]':null!=t.href?"/"+n+'[@href="'+t.href+'"]':null!=t.name?"/"+n+'[@name="'+t.name+'"]':null!=t.id?"/"+n+'[@id="'+t.id+'"]':null!=t.index?"/"+n:""}},u={limit:1e3,parseNode:function(e,t,r){if(void 0!==t){var i=function(e){var t=e.tagName.toLowerCase();if("input"===t)switch((n=e).type){case"password":return{type:"text",value:"***"};case"radio":return{type:"radio",value:n.value};case"checkbox":return{type:"checkbox",value:n.checked};case"file":return{type:"file",value:n.value};case"email":case"tel":case"url":case"number":case"search":case"text":case"date":case"datetime-local":case"week":case"month":case"color":return o.debug("Classified text node with value: "+n.value),{type:"text",value:n.value};case"submit":case"image":case"range":case"reset":return{type:n.type,value:void 0}}else{var n;if("textarea"===t)return{type:"text",value:(n=e).value};if("select"===t)return{type:"select",value:(n=e).value};if("a"===t)return{type:"a",value:(n=e).href};if(e.onclick)return}}(t),u=a.build(t,r,[]);if(void 0!==i)return n(n({},i),{time:e,path:c.build(u,t,i.type||"default")});o.debug("Could not classify node "+t)}}};function l(){return(new Date).getTime()}var o=i?{debug:console.debug,error:console.error}:{debug:function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t]},error:function(){for(var e=[],t=0;t<arguments.length;t++)e[t]=arguments[t]}},s=function(){function t(){var t=this;this.recordEvent=function(e){t.callback=e,t.list=[];var n=document.getElementById("inspector-frame")||document.createElement("div");n.id="inspector-frame",n.className="recorder",document.body.appendChild(n),document.addEventListener("change",t.recordChange,!0),document.addEventListener("click",t.recordClick,!0),document.addEventListener("keydown",t.handleKeyboardEvent)},this.recordChange=function(i){o.debug("Recording change event: ",i),(0,e.default)(i.target);var a=u.parseNode(l(),i.target,r);if(a)if("text"!==a.type||""!==a.value){var c=n(n({},a),{trigger:"change"});o.debug("Appending change action "+c),t.appendAction(c),(0,e.default)(t.callback),o.debug("Passing change event through callback"),t.callback({actionType:"event",list:t.list,url:document.URL}),o.debug("Succesfully invoked callback"),document.removeEventListener("change",t.recordChange,!0),document.removeEventListener("click",t.recordClick,!0)}else o.debug("Skipping saving empty text Change event");else o.debug("Skipping committing recordChange due to unclassified node")},this.recordClick=function(i){o.debug("Recording click event: ",i),(0,e.default)(i.target);var a=u.parseNode(l(),i.target,r);if(a)if("text"!==a.type||""!==a.value)if(a.type in["text","file","select"])o.debug("Skipping committing recordClick");else{var c=n(n({},a),{trigger:"click"});o.debug("Appending click action "+c),t.appendAction(c),(0,e.default)(t.callback),o.debug("Passing click event through callback"),t.callback({actionType:"event",list:t.list,url:document.URL}),o.debug("Succesfully invoked callback"),document.removeEventListener("change",t.recordChange,!0),document.removeEventListener("click",t.recordClick,!0)}else o.debug("Skipping saving empty text Click event");else o.debug("Skipping committing recordChange due to unclassified node")},this.handleKeyboardEvent=function(e){"Escape"===e.code&&t.stop()},this.stop=function(){t.callback&&t.callback({actionType:"stop",list:t.list,url:document.URL}),document.removeEventListener("change",t.recordChange,!0),document.removeEventListener("click",t.recordClick,!0);var e=document.getElementById("inspector-frame");e&&document.body.removeChild(e)},this.list=[];var i=document.getElementById("inspector-style")||document.createElement("style");i.id="inspector-style",i.type="text/css",document.head.appendChild(i)}return t.prototype.appendAction=function(e){var t=e;if(0!==this.list.length){var n=this.list[this.list.length-1];if(Math.abs(t.time-n.time)>20)return o.debug("Saving due to enough time between events"),void this.list.push(t);"click"!==t.trigger&&("change"!==t.trigger||"click"!==n.trigger?this.list.push(t):this.list[this.list.length-1]=t)}else this.list.push(t)},t}();window.Recorder=new s}();
},{"assert":"p+dr"}]},{},["WRAk"], null)