@font-face{font-family:Open Sans;font-style:normal;font-weight:400;src:local("Open Sans Regular"),local("OpenSans-Regular"),url(open-sans-v17-latin_cyrillic-regular.ff111bf4.woff) format("woff")}@font-face{font-family:Open Sans;font-style:italic;font-weight:400;src:local("Open Sans Italic"),local("OpenSans-Italic"),url(open-sans-v17-latin_cyrillic-italic.026dc2c1.woff) format("woff")}@font-face{font-family:Roboto Condensed;font-style:normal;font-weight:400;src:local("Roboto Condensed"),local("RobotoCondensed-Regular"),url(roboto-condensed-v18-latin_cyrillic-regular.db90b551.woff) format("woff")}